import org.gradle.api.DefaultTask
import org.gradle.api.file.FileCollection
import org.gradle.api.tasks.InputFiles
import org.gradle.api.tasks.OutputDirectory
import org.gradle.api.tasks.TaskAction
import org.gradle.api.tasks.incremental.IncrementalTaskInputs

class GenerateResourceConstants extends DefaultTask {

    File fromRoot

    @InputFiles
    FileCollection from

    @OutputDirectory
    File into

    def from(from) {
        this.fromRoot = project.file(from)
        this.from = project.fileTree(from) { include '**/*.properties' }
    }

    def into(into) {
        this.into = project.file(into)
    }

    @TaskAction
    void generateConstants(IncrementalTaskInputs inputs) {
        if (!inputs.incremental)
            project.delete(into.listFiles())

        inputs.outOfDate {
            createConstantClass(it.file)
        }
        inputs.removed {
            project.delete(constantClass(it.file).file)
        }
    }

    def constantClass(File resourceBundleFile) {
        String relativePath = fromRoot.toPath().relativize(resourceBundleFile.toPath()).toString()
        [
                file  : new File(into, relativePath.replaceAll(/\.properties$/, '.java')),
                pkg   : new File(relativePath).parent.replace(File.separator, '.'),
                name  : resourceBundleFile.name - '.properties',
                bundle: relativePath.replace(File.separator, '.') - ~/\.properties$/
        ]
    }

    String createConstantClass(File file) {
        PropertyResourceBundle rb = file.withReader { reader ->
            new PropertyResourceBundle(reader)
        }
        return generateResourceBundleClass(constantClass(file), rb)
    }

    void generateResourceBundleClass(constantClass, PropertyResourceBundle rb) {
        SortedSet<String> sortedKeys = rb.keys.toSet() as SortedSet
        project.mkdir(constantClass.file.parentFile)
        logger.info("generate $constantClass.file")

        StringBuilder buffer = new StringBuilder()
        constantClass.file.withPrintWriter() { PrintWriter printWriter ->
            fillClass(printWriter, constantClass, sortedKeys, rb)
        }
    }

    void fillClass(PrintWriter printWriter, constantClass, TreeSet<String> sortedKeys,
                   PropertyResourceBundle rb) {
        printWriter.println("package " + constantClass.pkg + ";")
        printWriter.println()
        printWriter.println("public class " + constantClass.name)
        printWriter.println("{")
        printWriter.println("    public static final String RESOURCEBUNDLE = \"" + constantClass.bundle + "\";")
        printWriter.println()
        for (String key in sortedKeys) {
            printWriter.println("    /** " + rb.getString(key) + " */")
            printWriter.println("    public static final String " + key + " = \"" + key + "\";")
        }
        printWriter.println()
        printWriter.println("}")
    }

}
