import org.gradle.api.DefaultTask
import org.gradle.api.file.FileCollection
import org.gradle.api.tasks.InputFiles
import org.gradle.api.tasks.OutputFile
import org.gradle.api.tasks.TaskAction

class GenerateResourceConstantsIndex extends DefaultTask {

    File fromRoot

    @InputFiles
    FileCollection from

    @OutputFile
    File into

    def from(from) {
        this.fromRoot = project.file(from)
        this.from = project.fileTree(from) { include '**/*.properties' }
    }

    def into(into) {
        this.into = project.file(into)
    }

    @TaskAction
    void generateConstantsIndex() {
        into.withPrintWriter { writer ->
            from.collect { constantClass(it).bundle }.each {
                writer.println it
            }
        }
    }

    def constantClass(File resourceBundleFile) {
        String relativePath = fromRoot.toPath().relativize(resourceBundleFile.toPath()).toString()
        [
                file  : new File(into, relativePath.replaceAll(/\.properties$/, '.java')),
                pkg   : new File(relativePath).parent.replace(File.separator, '.'),
                name  : resourceBundleFile.name - '.properties',
                bundle: relativePath.replace(File.separator, '.') - ~/\.properties$/
        ]
    }

}
